import sys
import logging
from valeriovive.nano import Nano
from valeriovive.scheduler import Scheduler,dict_from_json

import logging.config
import sys, time, os
from valeriovive.daemon import Daemon

#!/usr/bin/env python


class ExperimentDaemon(Daemon):
    def run(self):
        experiment=Experiment(self.args)
        experiment.start_experiment()

def ExperimentShell():
    Experiment.run_shell()


def ExperimentDebug(args):
    experiment=Experiment(args)
    experiment.oneshot()

class Experiment(Scheduler):
    """
    Experiment classs
    controls the evolution of the experiment with the Scheduler
    Each parameter can be set through this class
    As many config file as many bots will be created, the bot are stored
    in the 'bots' attribute.
    """
    def __init__(self,*args, **kwargs):
        self.log = logging.getLogger("root")
        Scheduler.__init__(self, self.events_manager)
        self.bots=[]
        if "conf" in kwargs:
            self.prepare_experiment(kwargs["conf"])
        if "remove_friends" in kwargs:
            self.remove_friends=True
        self.log.debug("Experiment configured with {}".format(self.conf_file ))


    @classmethod
    def create_experiment(cls, **kwargs):
        return Experiment(kwargs)


    """
    Prepare experiment with conf_file params
    """
    def prepare_experiment(self, conf_file=None):

        ##set the conf file or use the default
        if isinstance(conf_file, str):
            self.conf = dict_from_json(conf_file)
            self.conf_file = os.path.join(os.getcwd(),conf_file)
        else:
            self.log.error("if you re not setting conf file, you are wrong")
            self.conf_file = os.path.join(os.getcwd(),'valeriovive/nani.json')
            self.conf = dict_from_json(self.conf_file)
            self.log.debug("conf file is {}".format(self.conf_file))

        ##pass the conf file to each agent
        for agent in self.conf['agents']:
            nano = Nano(agent, self.conf_file)
            self.bots.append(nano)

    @staticmethod
    def run_shell():
        settings={}

        while True:
            s = input()
            if s == "start":
                exp = Experiment.create_experiment(settings)
            elif s == "set conf":
                print("digit the path to conf file")
                conf_path = input()
                try:
                    settings["conf"]=conf_path
                except:
                    print("set configuration failed")
            elif s == "remove friends":
               settings["remove_friends"]=True
            elif s == "exit" or s == "quit" or s == "q":
                # break

    def start_experiment(self, duration=0):
        '''
        Start the experiment.
        Argument:
        duration -- expressed in seconds
        if duration not set, it will continue indefinitely
        '''
        self.log.debug("Experiment routine starting")
        self.routine_manager(self.conf_file)

    def oneshot(self):
        """
        Play one only experience for the bot"
        """
        self.log.debug("Experiment one shot")
        for nano in self.bots:
            if self.remove_friends:
                nano.remove_friends()
            else:
                nano.set_behaviour()
                nano.online_experience()

    @property
    def common_hours(self):
        return self.conf['common']['hours']

    def events_manager(self):
        """
        Do the relevant action for the experiment:
        1) For each nano, at scheduled time, start online experience
        """
        for nano in self.bots:
            nano.set_behaviour()
            ##retrieve action hours
            try:
                nano.events = Scheduler.events_from_hours(nano.get_hours())
            except:
                nano.events = Scheduler.events_from_hours(self.common_hours)
            ##schedule actions
            for event_time in nano.events:
                self.schedule_event(event_time, nano.online_experience)
                logging.debug("nano {} scheduled at {}".format(nano.name, str(event_time)))



