from valeriovive.twitter_agent import TwitterAgent
import  random
import json
import logging
import time
from valeriovive.scheduler import dict_from_json


class Nano:
    '''
    Create Nano Object
    Nano is the active element of the experiment,it does stuff
    Nano remains during all the experiment.
    Nano is actually the bot and will be initialited with conf.file path. That cannot change.
    Nano has no perception of time.

    Attributes
    ==========
    CONF:
    common_conf dict  See README
    own_conf    dict  See README

    __twitter_agent TwitterAgent(Tweepy)

    Behaviour:
    actions         set of __twitter agent methods
    buddy_list      list(strigs)
    hash_list
    events          list(hours(int[0-23]))

    Methods
    ===========


    '''
    def __init__(self, name, conf_file):
        self.name=name
        self.conf_file=conf_file
        self.twitter_agent = TwitterAgent(name,dict_from_json(self.conf_file)[self.name]["twitter"])
        self.actions =[]
        ###
        self.buddy_list = []
        self.hash_list  = []
        self.actions_list =[]

        self.events=None

        ### loggers
        self.logger = logging.getLogger(name)
        self.logger.info('created Nano {} obj, and file {}'.format(self.name,conf_file))

    @property
    def conf(self):
        return dict_from_json(self.conf_file)[self.name]

    # def twitter_conf(conf_file):
        # try:
            # return dict_from_json(conf_file)[self.name]["twitter"]
        # except KeyError:
            # raise ValueError("missing twitter conf in the configuration file")
    """
    Set behaviour

    Dict needs to contain buddies , hashtags , actions
    """
    def set_behaviour(self):
        if self.buddy_list == dict_from_json(self.conf_file)["individual_behaviour"]:
            self.buddy_list = dict_from_json(self.conf_file)[self.name]['buddies']
            self.hash_list  = dict_from_json(self.conf_file)[self.name]['hashtags']
            self.actions_list    = dict_from_json(self.conf_file)[self.name]['actions']
        else:
            self.buddy_list = dict_from_json(self.conf_file)["common"]['buddies']
            self.hash_list  = dict_from_json(self.conf_file)["common"]['hashtags']
            self.actions_list    = dict_from_json(self.conf_file)["common"]['actions']
        self.logger.debug("buddies: {} \n, hashtags: {} \n, actions: {}\n".format(self.buddy_list,self.hash_list,self.actions_list))
        self.get_actions()

    def get_hours(self):
        return dict_from_json(self.conf_file[self.name]['hours'])

    def get_actions(self):
        """
        Retrieve functions from config and add to actions_list
        """
        self.actions=[]
        if "search_and_follow" in self.actions_list:
            self.actions.append(self.twitter_agent.search_and_follow)
        if "search_and_retweet" in self.actions_list:
            self.actions.append(self.twitter_agent.search_and_retweet)
        if "get_user_timeline" in self.actions_list:
            self.actions.append(self.twitter_agent.get_user_timeline)
        if "destroy_all_friends" in self.actions_list:
            self.actions.append(self.twitter_agent.destroy_all_friends)

        # if "retweet_my_friends" in self.actions:
            # self.actions.append(self.__twitter_agent.retweet_my_friends)

    """
    Start Online Session
    log with tweepy and do stuff as in behaviour
    """
    def online_experience(self):
        random.shuffle(self.actions)
        self.twitter_agent.connect()
        if not self.twitter_agent.connected:
            self.logger.error("cannot connect to Twitter!")
            return
        # self.twitter_agent.get_user_timeline()
        for action in self.actions:
            self.logger.debug("next action is {}".format(action.__name__))
            try:
                action(self.buddy_list, self.hash_list)
            except :
                self.logger.exception("Action {} failed".format(action.__name__))
        if 'dbpath' in self.conf:
            with open(self.conf['dbpath'], 'a') as buf:
                record = {}
                record['actions'] = [a.__name__ for a in self.actions]
                record['buddy']= self.buddy_list
                record['hash'] = self.hash_list
                record['time'] = int(time.time())
                buf.write(json.dumps(record))
                buf.write("\n")

    def remove_friends(self):
        self.twitter_agent.connect()
        try:
            self.twitter_agent.destroy_all_friends()
        except :
            self.logger.error("cannot connect to Twitter!")




